// this is a small sdk to enable you to use my tools in your own tools !

#if defined(_MSC_VER)
#pragma warning(disable:4115)
#endif // _MSC_VER

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#undef WIN32_LEAN_AND_MEAN

#include <stdio.h>  // prinf 
#include <string.h> // sprintf

#if defined(__cplusplus)
extern "C" {
#endif // __cplusplus

// get version
#define GETCCFVERSION            "GetCCFVersion"
typedef char* (*GetCCFVersionFunctionPtr)(void);

// set display string callback
#define SETDISPLAYSTRINGCALLBACK "SetDisplayStringCallback"
typedef void (WINAPI *DISPLAYSTRINGCALLBACK)(char *);
typedef void (*SetDisplayStringCallbackFunctionPtr) (DISPLAYSTRINGCALLBACK DisplayStringCallBack);

// run ccfdumper
#define CCFRUNDUMPER             "CCFRunDumper"
typedef int (*CCFRunDumperFunctionPtr)(char *szInputCCF,char *szOutputDirectory,int DumpFlags);
// DumpFlags masks : can be combined like CCF_DUMP_ALL
#define CCF_DUMP_FONT      0x00000001
#define CCF_DUMP_STRINGS   0x00000002
#define CCF_DUMP_ICONS     0x00000004
#define CCF_DUMP_PANELS    0x00000008
#define CCF_DUMP_CODES     0x00000010
#define CCF_DUMP_SUBPANELS 0x00000020
#define CCF_DUMP_ALL (CCF_DUMP_FONT | CCF_DUMP_STRINGS | CCF_DUMP_ICONS | CCF_DUMP_PANELS | CCF_DUMP_CODES | CCF_DUMP_SUBPANELS)

// run ccfdecompiler
#define CCFRUNDECOMPILER          "CCFRunDecompiler"
#define CCF_OUTPUT_FORMAT_CSF                1
#define CCF_OUTPUT_FORMAT_XML                2
#define CCF_OUTPUT_FORMAT_GENERATE_BUTTON_ID 8
typedef int (*CCFRunDecompilerFunctionPtr)(char *szInputCCF,char *szOutputCSF,int OutputFormatFlags);

// run ccfcompiler
#define CCFRUNCOMPILER            "CCFRunCompiler"
#define CCF_COMPILER_FLAG_COMPRESS_ICON  1
typedef int (*CCFRunCompilerFunctionPtr)(char *szInputCSF,char *szOutputCCF,int Flags);

// ccfircodes : not available for the moment
#define CCFRUNIRCODES             "CCFRunIrCodes"
typedef int (*CCFRunIrCodesFunctionPtr)(char *szFormat,char *szOutputCCF,int Device,int Frequency,int Special);
// szFormat : name as in CCFIrCode.ini

#if defined(__cplusplus)
}
#endif // __cplusplus

// display string callback
void WINAPI DisplayStringCallback(char *str)
{
  printf(str);
}

#if defined(_DEBUG)
  #define CCFDLL "CCFDllD.dll"
#else
  #define CCFDLL "CCFDll.dll"
#endif // _DEBUG

void CCFSDK_Test()
{
  // load ccfdll.dll
  HMODULE hCCFDll=LoadLibrary(CCFDLL);
  if (!hCCFDll)
  {
    DisplayStringCallback("Can't find " CCFDLL ".\n");
  }
  else
  {
    // function pointers
    GetCCFVersionFunctionPtr GetCCFVersion=NULL;
    SetDisplayStringCallbackFunctionPtr SetDisplayStringCallback=NULL;
    CCFRunDumperFunctionPtr             CCFRunDumper=NULL;
    CCFRunDecompilerFunctionPtr         CCFRunDecompiler=NULL;
    CCFRunCompilerFunctionPtr           CCFRunCompiler=NULL;
    CCFRunIrCodesFunctionPtr            CCFRunIrCodes=NULL;

    int ret=0;      

    // define callback for displaying all messages during dumping,compiling,decompiling etc...
    SetDisplayStringCallback=(SetDisplayStringCallbackFunctionPtr) GetProcAddress(hCCFDll,SETDISPLAYSTRINGCALLBACK);
    if (SetDisplayStringCallback)
    {
      SetDisplayStringCallback(DisplayStringCallback);
    }

    // call getversion function : 
    GetCCFVersion=(GetCCFVersionFunctionPtr) GetProcAddress(hCCFDll,GETCCFVERSION);
    if (GetCCFVersion)
    {
      char szVersion[256]={0};
      sprintf(szVersion,"%s loaded\n",GetCCFVersion());
      DisplayStringCallback(szVersion);
    }

    // call dump function : it dumps CCFSDK.ccf file into CCFSDK_htm directory
    CCFRunDumper=(CCFRunDumperFunctionPtr) GetProcAddress(hCCFDll,CCFRUNDUMPER);
    if (CCFRunDumper)
    {
      ret=CCFRunDumper("CCFSDK.ccf","CCFSDK_htm",CCF_DUMP_ALL);
      if (!ret)
      {
        // error
      }
    }

    // call decompiler function : it decompiles CCFSDK.ccf file to CCFSDK.csf file (& bmp directory)
    CCFRunDecompiler=(CCFRunDecompilerFunctionPtr) GetProcAddress(hCCFDll,CCFRUNDECOMPILER);
    if (CCFRunDecompiler)
    {
      ret=CCFRunDecompiler("CCFSDK.ccf","CCFSDK.csf",CCF_OUTPUT_FORMAT_CSF);
      if (!ret)
      {
        // error
      }

      ret=CCFRunDecompiler("CCFSDK.ccf","CCFSDK.xml",CCF_OUTPUT_FORMAT_XML);
      if (!ret)
      {
        // error
      }

    }

    // call compiler function : it compiles CCFSDK.csf file (& bmp directory) to CCFSDKGenerated.csf file
    CCFRunCompiler=(CCFRunCompilerFunctionPtr) GetProcAddress(hCCFDll,CCFRUNCOMPILER);
    if (CCFRunCompiler)
    {
      // to activate icon compression
      int EnableCompression=0;
      int Flags=(EnableCompression)?CCF_COMPILER_FLAG_COMPRESS_ICON:0;

      ret=CCFRunCompiler("CCFSDK.csf","CCFSDKGeneratedFromCSF.ccf",Flags);
      if (!ret)
      {
        // error
      }
      ret=CCFRunCompiler("CCFSDK.xml","CCFSDKGeneratedFromXML.ccf",Flags);
      if (!ret)
      {
        // error
      }
    }

    
    // call ircodes function : it generates Sony12-Device1.ccf file with panels containing ir codes for device 1 in sony12 format
    CCFRunIrCodes=(CCFRunIrCodesFunctionPtr) GetProcAddress(hCCFDll,CCFRUNIRCODES);
    if (CCFRunIrCodes)
    { 
      /*
      ret=CCFRunIrCodes("Sony12","Sony12-Device1.ccf",1,40000,0);
      if (!ret)
      {
        // error
      }
      */
    }

    // unload ccfdll.dll
    FreeLibrary(hCCFDll);
  }
}

void main()
{
  CCFSDK_Test();
}